#include <stdint.h>

/*Boot cfg registers*/
#define KICK0			(*(volatile int*)(0x02620038))
#define KICK1			(*(volatile int*)(0x0262003C))
#define MAINPLLCTL0		(*(volatile int*)(0x02620328))
#define MAINPLLCTL1		(*(volatile int*)(0x0262032C))
#define DDR3PLLCTL1		(*(volatile int*)(0x02620334))
#define PAPLLCTL1		(*(volatile int*)(0x0262033C))

/*PLL controller registers*/
#define PLLCTL			(*(volatile int*)(0x02310100))
#define SECCTL			(*(volatile int*)(0x02310108))
#define PLLM			(*(volatile int*)(0x02310110))
#define PLLCMD			(*(volatile int*)(0x02310138))
#define PLLSTAT			(*(volatile int*)(0x0231013C))

#define PLLDIV1			(*(volatile int*)(0x0231011C))
#define PLLDIV4			(*(volatile int*)(0x02310164))
#define PLLDIV7			(*(volatile int*)(0x02310170))

void Init_PLL(uint32_t PLLM_val, uint32_t PLLDIV1_val, uint32_t PLLDIV4_val, uint32_t PLLDIV7_val)
{
	int i;
	
	/*Unlock Boot Config*/
	KICK0 = 0x83E70B13;
	KICK1 = 0x95A4F1E0;
	
	SECCTL = 0x00890000;
	/*In PLLCTL, write PLLENSRC = 0 (enable PLLEN bit)*/
	PLLCTL &= ~(0x00000020);
	
	/*In PLLCTL, write PLLEN = 0 (bypass mode)*/
	PLLCTL &= ~(0x00000001);
	
	/*Wait 4 cycles for the slowest of PLLOUT or reference clock source (CLKIN)*/
	for(i=0;i<100;i++);
	
	/*In PLLCTL, write PLLRST = 1 (PLL is reset) */
	PLLCTL |= 0x00000008;
	
	/*If necessary program PLLM*/
	PLLM = PLLM_val*2 -1;
	
	/*Program BWADJ[7:0] in MAINPLLCTL0*/
	MAINPLLCTL0 |= (PLLM_val << 23) & 0xFF000000;  /*BWADJ[7:0] = multiplier value/2 */
	MAINPLLCTL0 &= (PLLM_val << 23) | 0x00FFFFFF;
	/*If necessary program PLLDIV1n.  Note that you must aplly the GO operation
	 to change these dividers to new ratios*/
	 
	 /*Check that the GOSTAT bit in PLLSTAT is cleared to show that no GO
	 operation is currently in progress*/
	 while((PLLSTAT) & 0x00000001);
	 
	 /*Program the RATIO field in PLLDIVn to the desired new divide-down rate.
	 If RATIO field is changed, the PLL controller will flag the change in the
	 corresponding bit of DCHANGE*/
	 PLLDIV1 = (PLLDIV1_val-1) | 0x8000;  //Set PLLDIV1
	 PLLDIV4 = (PLLDIV4_val-1) | 0x8000;  //Set PLLDIV4
	 PLLDIV7 = (PLLDIV7_val-1) | 0x8000;  //Set PLLDIV7
	 
	 /*Set GOSET bit in PLLCMD to initiate the GO operation to change the divide
	 values and align the SYSCLKs as programmed */
	 PLLCMD |= 0x00000001;
	 
	 /*Read the GOSTAT bit in PLLSTAT to make sure the bit returns to 0 to
	 indicate that the GO operation has completed */
	 while((PLLSTAT) & 0x00000001);
	 
	 /*Wait for PLL to properly reset (128 CLKIN1 cycles) */
	 for(i=0;i<1000;i++);
	 
	 /*In PLLCTL, write PLLRST = 0 to bring PLL out of reset */
	 PLLCTL &= ~(0x00000008);
	 
	 /*Wait for PLL to lock (2000 CLKIN1 cycles) */
	 for(i=0;i<5000;i++);
	 
	 SECCTL = 0x00090000;
	 /*In PLLCTL, write PLLEN = 1 to enable PLL mode */
	 PLLCTL = 0x00000041;
	 
	 for(i=0;i<4000;i++);
}

